#!/usr/bin/env python
# -*- coding: utf-8 -*-

import struct
import atexit
import tempfile
import syslog
import subprocess
import sys, os, time
from signal import SIGTERM


daemonrunning = True

class Daemon:
    def __init__(self, pidfile, stdin='/dev/null', stdout='/dev/null', stderr='/dev/null'):
        self.stdin = stdin
        self.stdout = stdout
        self.stderr = stderr
        self.pidfile = pidfile

    def daemonize(self):
        try:
            pid = os.fork()
            if pid > 0:
                sys.exit(0) # exit first parent
        except OSError, e:
            sys.stderr.write("fork #1 failed: %d (%s)\n" % (e.errno, e.strerror))
            sys.exit(1)

        os.chdir("/")
        os.setsid()
        os.umask(0)

        # do second fork
        try:
            pid = os.fork()
            if pid > 0:
                sys.exit(0) # exit from second parent
        except OSError, e:
            sys.stderr.write("fork #2 failed: %d (%s)\n" % (e.errno, e.strerror))
            sys.exit(1)

        # redirect standard file descriptors
        sys.stdout.flush()
        sys.stderr.flush()
        si = file(self.stdin, 'r')
        so = file(self.stdout, 'a+')
        se = file(self.stderr, 'a+', 0)
        os.dup2(si.fileno(), sys.stdin.fileno())
        os.dup2(so.fileno(), sys.stdout.fileno())
        os.dup2(se.fileno(), sys.stderr.fileno())

        # write pidfile
        atexit.register(self.delpid)
        pid = str(os.getpid())
        file(self.pidfile,'w+').write("%s\n" % pid)

    def delpid(self):
        os.remove(self.pidfile)

    def start(self):
        try:
            pf = file(self.pidfile,'r')
            pid = int(pf.read().strip())
            pf.close()
        except IOError:
            pid = None

        if pid:
            message = "pidfile %s already exist. Daemon already running?\n"
            sys.stderr.write(message % self.pidfile)
            sys.exit(1)

        # Start the daemon
        self.daemonize()
        self.run()

    def stop(self):
        daemonrunning = False

        try:
            pf = file(self.pidfile,'r')
            pid = int(pf.read().strip())
            pf.close()
        except IOError:
            pid = None

        if not pid:
            message = "pidfile %s does not exist. Daemon not running?\n"
            sys.stderr.write(message % self.pidfile)
            return # not an error in a restart

        # Try killing the daemon process
        try:
            while 1:
                os.kill(pid, SIGTERM)
                time.sleep(0.1)
        except OSError, err:
            if os.path.exists(self.pidfile):
                os.remove(self.pidfile)

    def restart(self):
        self.stop()
        self.start()

    def run(self):
        """
        You should override this method when you subclass Daemon.
        """


class ApkKeyNotify:
    def __init__(self):
        # default: /dev/input/event3
        self.infile_path = "/dev/input/event3"
        # states
        self.tpad_btn = 1
        self.pwm_btn = 1.0
        #long int, long int, unsigned short, unsigned short, unsigned int
        self.FORMAT = 'llHHI'
        self.EVENT_SIZE = struct.calcsize(self.FORMAT)

        os.system("echo 1 > /sys/class/leds/pwm_g:green/brightness")
        os.system("echo 0 > /sys/class/leds/pwm_r:yellow/brightness")
        os.system("echo 0 > /sys/class/leds/tpad_off:red/brightness")

        #open file in binary mode
        self.in_file = open(self.infile_path, "rb")

    def check(self):
        event = self.in_file.read(self.EVENT_SIZE)
        if event:
            (tv_sec, tv_usec, type, code, value) = struct.unpack(self.FORMAT, event)

            ## "wifi_btn" - <238> KEY_WLAN
            #if type == 1 and code == 238 and value == 0:
            #    print("wifi_btn")

            ## "avia_btn" - <247> KEY_RFKILL
            #if type == 1 and code == 247 and value == 0:
            #    print("avia_btn")

            # "pwm_btn" - <243> KEY_BRIGHTNESS_CYCLE
            if type == 1 and code == 243 and value == 0:
                #syslog.syslog(syslog.LOG_INFO, "pwm_btn - <243> KEY_BRIGHTNESS_CYCLE")
                self.pwm_btn -= 0.2
                if self.pwm_btn <= 0.2:
                    self.pwm_btn = 1.0
                outs, errs = subprocess.Popen(['/usr/bin/xrandr','--output','DisplayPort-1','--brightness',str(self.pwm_btn)], stdout=subprocess.PIPE, stderr=subprocess.PIPE,
                             env={'DISPLAY':':0','XAUTHORITY':'/root/.Xauthority'}).communicate()
                if errs:
                    syslog.syslog(syslog.LOG_ERR, errs)
                if self.pwm_btn >= 1.0:
                    os.system("echo 1 > /sys/class/leds/pwm_g:green/brightness")
                    os.system("echo 0 > /sys/class/leds/pwm_r:yellow/brightness")
                elif self.pwm_btn <= 0.4:
                    os.system("echo 0 > /sys/class/leds/pwm_g:green/brightness")
                    os.system("echo 1 > /sys/class/leds/pwm_r:yellow/brightness")
                else:
                    os.system("echo 0 > /sys/class/leds/pwm_g:green/brightness")
                    os.system("echo 0 > /sys/class/leds/pwm_r:yellow/brightness")

            # "tpad_btn" - <0x212> KEY_TOUCHPAD_TOGGLE
            elif type == 1 and code == 0x212 and value == 0:
                if self.tpad_btn == 1:
                    #syslog.syslog(syslog.LOG_INFO, "tpad_btn - <0x212> KEY_TOUCHPAD_TOGGLE - OFF")
                    os.system("echo 1 > /sys/class/leds/tpad_off:red/brightness")
                    outs, errs = subprocess.Popen(['/usr/bin/xinput','disable','Mouse1'], stdout=subprocess.PIPE, stderr=subprocess.PIPE,
                                 env={'DISPLAY':':0','XAUTHORITY':'/root/.Xauthority'}).communicate()
                    if errs:
                        syslog.syslog(syslog.LOG_ERR, errs)
                    self.tpad_btn = 0
                else:
                    #syslog.syslog(syslog.LOG_INFO, "tpad_btn - <0x212> KEY_TOUCHPAD_TOGGLE - ON")
                    os.system("echo 0 > /sys/class/leds/tpad_off:red/brightness")
                    outs, errs = subprocess.Popen(['/usr/bin/xinput','enable','Mouse1'], stdout=subprocess.PIPE, stderr=subprocess.PIPE,
                                 env={'DISPLAY':':0','XAUTHORITY':'/root/.Xauthority'}).communicate()
                    if errs:
                        syslog.syslog(syslog.LOG_ERR, errs)
                    self.tpad_btn = 1

            # "cover_down" - <245> KEY_DISPLAY_OFF
            elif type == 1 and code == 245 and value == 0:
                #syslog.syslog(syslog.LOG_INFO, "cover_down - <245> KEY_DISPLAY_OFF")
                outs, errs = subprocess.Popen(['/usr/bin/xset','dpms','force','off'], stdout=subprocess.PIPE, stderr=subprocess.PIPE,
                             env={'DISPLAY':':0','XAUTHORITY':'/root/.Xauthority'}).communicate()
                if errs:
                    syslog.syslog(syslog.LOG_ERR, errs)

            # "cover_down(up)" - <245> KEY_DISPLAY_OFF
            elif type == 1 and code == 245 and value == 1:
                #syslog.syslog(syslog.LOG_INFO, "cover_down(up) - <245> KEY_DISPLAY_OFF")
                outs, errs = subprocess.Popen(['/usr/bin/xset','dpms','force','on'], stdout=subprocess.PIPE, stderr=subprocess.PIPE,
                             env={'DISPLAY':':0','XAUTHORITY':'/root/.Xauthority'}).communicate()
                if errs:
                    syslog.syslog(syslog.LOG_ERR, errs)

            # "Power-key" - <116> KEY_POWER
            elif type == 1 and code == 116 and value == 0:
                syslog.syslog(syslog.LOG_INFO, "Power-key - <116> KEY_POWER pressed")
                os.system("halt")

    def finish(self):
        self.in_file.close()


class ApkKeyNotifyDaemon(Daemon):
    def run(self):
        aknd = ApkKeyNotify()
        while daemonrunning:
            aknd.check()
        aknd.finish()


if __name__ == '__main__':
    pidFile = tempfile.gettempdir() + '/daemonApkKeyNotify.pid'
    daemon = ApkKeyNotifyDaemon(pidFile)
    if len(sys.argv) == 2:
        if 'start' == sys.argv[1]:
            daemon.start()
        elif 'stop' == sys.argv[1]:
            daemon.stop()
        elif 'restart' == sys.argv[1]:
            daemon.restart()
        else:
            sys.exit(2)
        sys.exit(0)
    else:
        print 'usage: %s start|stop|restart' % sys.argv[0]
        sys.exit(2)
